package MN::News; # class to manage MyNews news

use strict;

@MN::News::ISA = qw(MN::Object);

use MN::Object;
use MN::Topic;
use HTTP::Lite;

my $ms = 0;
my @tags = ();

sub new # constructor
{
    my $class_name = shift;

    my $self = new MN::Object();
    
    bless $self, $class_name;

    return $self; # new News object
}
sub getClass # get database table for the class 
{
    return "mnnews"; # "mnnews" else
}

sub getKey # get database key for the class
{
    return "id"; # "id"
}

sub getFields # get database fields for the class
{
    return { # fields hash table
	"id" => "string",
	"name" => "string",
	"date" => "date",
	"topic" => "string",
	"body" => "string",
	"template" => "string",
	"crtime" => "string",
    };
}

sub getNewsId # get the news id (without template name)
{
    my $self = shift;

    my $id = $self->{id};
    $id =~ s/^[^_]+_//;
    return $id; # the news id
}

sub getTemplate # get the news template
{
    my $self = shift;

    return  MN::Template->getTemplate($self->{template});
}

sub getTopic # get the news topic
{
    my $self = shift;

    return  MN::Topic->getTopic($self->{topic});
}

sub extract # extract news data from news lines
{
    my $self = shift;
    my $idonly = shift; # 1 to only extract news id

    my $line;
    my $tmpl = $self->getTemplate();
    my $active = 0;
    my $first;
    my $last;

    my $ss = $tmpl->getSearchStrings();
    my $search;
    my $replace;

    if ($self->{entry}) # atom entry
    {
	my $field;
	$field = $self->{entry}->query("id");
	if ($field)
	{
	    $self->{guid} = $field->text_value();
	}
	$self->{link} = $self->{entry}->query('link/@href');
	$field = $self->{entry}->query("title");
	if ($field)
	{
	    $self->{name} = MN::CGI->unutf($field->text_value());
	}
	$field = $self->{entry}->query("content");
	if ($field)
	{
	    $self->{body} = MN::CGI->unutf($field->text_value());
	}
	else
	{
	    $field = $self->{entry}->query("summary");
	    if ($field)
	    {
		$self->{body} = MN::CGI->unutf($field->text_value());
	    }
	}
	$self->{body} =~ s/$/<br>/mg;
	$field = $self->{entry}->query("modified") || $self->{entry}->query("issued");
	if ($field && $field->text_value() =~ /(\d+\-\d+\-\d+)/)
	{
	    $self->{date} = $1;
	}
	$field = $self->{entry}->query("dc:subject");
	if ($field)
	{
	    $self->{topicname} =  MN::CGI->unutf($field->text_value());
	    $self->{topic} = lc($self->{topicname});
	    $self->{topic} =~ s/\s//g;
	    $self->{topic} = "$tmpl->{id}_$self->{topic}";
	}
	my $id = $self->{guid} || $self->{link};
	if ($tmpl->{rx_news_id} && $id =~ /$tmpl->{rx_news_id}/)
	{
	    $self->{id} = $tmpl->{id}."_".$1;		    
	}
	else
	{
	    $self->{id} = $tmpl->{id}."_".$id;
	}
	if ($tmpl->{rx_topic_id} && $id =~ /$tmpl->{rx_topic_id}/)
	{
	    $self->{topic} = $1;
	    $self->{topicname} = $self->{topic};
	}
    }
    elsif ($self->{item}) # rss item
    {
	my $field;
	my $fname;
	foreach $field ($self->{item}->children())
	{
	    $fname = $field->{name};
	    $fname =~ s/(.*\/)//;
	    if ($fname eq 'guid')
	    {
		$self->{guid} = $field->{value};
	    }
	    elsif ($fname eq 'link')
	    {
		$self->{link} = $field->{value};
	    }
	    elsif (!$idonly)
	    {
		if ($fname eq 'title')
		{
		    $self->{name} = MN::CGI->unutf($field->{value});
		}
		elsif ($fname eq 'description')
		{
		    $self->{body} =  MN::CGI->unutf($field->{value});
		    $self->{body} =~ s/$/<br>/mg;
		}
		elsif ($fname eq 'pubDate' && $field->{value} =~ /^\w+\,\s*(\d+)\s*(\w+)\s*(\d+)/)
		{
		    my $d = $1;
		    my $mn = $2;
		    my $y = $3;
		    my %mi = (
			      'Jan' => 1,
			      'Feb' => 2,
			      'Mar' => 3,
			      'Apr' => 4,
			      'May' => 5,
			      'Jun' => 6,
			      'Jul' => 7,
			      'Aug' => 8,
			      'Sep' => 9,
			      'Oct' => 10,
			      'Nov' => 11,
			      'Dec' => 12,
			      );
		    my $m = $mi{$mn};
		    $self->{date} = sprintf("%04d-%02d-%02d", $y, $m, $d);
		}
		elsif ($fname eq 'date' && $field->{value} =~ /^(\d+\-\d+\-\d+)/)
		{
		    $self->{date} = $1;
		}
		elsif ($fname eq 'category' || $fname eq 'subject')
		{
		    $self->{topicname} =  MN::CGI->unutf($field->{value});
		    $self->{topic} = lc($self->{topicname});
		    $self->{topic} =~ s/\s//g;
		    $self->{topic} = "$tmpl->{id}_$self->{topic}";
		}
	    }
	}
	my $id = $self->{guid} || $self->{link};
	if ($tmpl->{rx_news_id} && $id =~ /$tmpl->{rx_news_id}/)
	{
	    $self->{id} = $tmpl->{id}."_".$1;		    
	}
	else
	{
	    $self->{id} = $tmpl->{id}."_".$id;
	}
	if ($tmpl->{rx_topic_id} && $id =~ /$tmpl->{rx_topic_id}/)
	{
	    $self->{topic} = $1;
	    $self->{topicname} = $self->{topic};
	}
    }
    else
    {
	if (!$tmpl->{rx_news_id})
	{
	    die "Template $tmpl->{id} : no rx_news_id value";
	}
	
	if ($tmpl->{rx_news_first})
	{
	    $first = $tmpl->{rx_news_first};
	}
	else
	{
	    $first = $tmpl->{rx_news_start};
	}
	if ($tmpl->{rx_news_last})
	{
	    $last = $tmpl->{rx_news_last};
	}
	else
	{
	    $last = $tmpl->{rx_news_end};
	}
	
	foreach $line (@{$self->{lines}})
	{
	    if ($line =~ /$tmpl->{rx_news_id}/)
	    {
		$self->{id} = "$tmpl->{id}_$1";
		if ($idonly)
		{
		    return $self->{id};
		}
	    }
	    if ($idonly)
	    {
		next;
	    }
	    if ($tmpl->{rx_news_name} && $line =~ /$tmpl->{rx_news_name}/)
	    {
		$self->{name} = $1;
		$self->{name} =~ s/<[^>]+>//g;
		$self->{name} =~ s/&nbsp;/ /g;
	    }
	    my $rxdate = $tmpl->{rx_news_date};
	    my $daterev = $rxdate =~ s/^~//;
	    if ($tmpl->{rx_news_date} && $line =~ /$rxdate/)
	    {
		if ($daterev)
		{
		    $self->{date} = sprintf("20%02d-%02d-%02d", $3, $2, $1);
		}
		else
		{
		    $self->{date} = sprintf("20%02d-%02d-%02d", $1, $2, $3);
		}
	    }
	    if ($tmpl->{rx_topic_id} && $line =~ /$tmpl->{rx_topic_id}/)
	    {
		$self->{topic} = "$tmpl->{id}_$1";
	    }
	    if ($tmpl->{rx_topic_name} && $line =~ /$tmpl->{rx_topic_name}/)
	    {
		$self->{topicname} = $1;
	    }
	    if ($tmpl->{rx_topic_icon} && $line =~ /$tmpl->{rx_topic_icon}/)
	    {
		$self->{topicicon} = $1;
	    }
	    if ($first && $line =~ /$first/)
	    {
		$line = $1 || "";
		foreach $search (@{$ss})
		{
		    $replace = $tmpl->{replace}->{$search};
		    $line =~ s/$search/$replace/g;
		}
		if ($line =~ /\S/)
		{
		    $self->{body} = "$line\n";
		}
		$active = 1;
		next;
	    }
	    if ($last && $line =~ /$last/)
	    {
		$line = $1 || "";
		foreach $search (@{$ss})
		{
		    $replace = $tmpl->{replace}->{$search};
		    $line =~ s/$search/$replace/g;
		}
		if ($line =~ /\S/)
		{
		    $self->{body} .= $line;
		}
		$active = 0;
	    }
	    if ($active)
	    {
		foreach $search (@{$ss})
		{
		    $replace = $tmpl->{replace}->{$search};
		    $line =~ s/$search/$replace/g;
		}
		if ($line =~ /\S/)
		{
		    $self->{body} .= "$line\n";
		}
	    }
	}
    }
}

sub getTopicIcon # extract topic icon from template website
{
    my $self = shift;

    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    my $imgdir;
    if ($mne->{conf}->{webdir})
    {
	$imgdir = "$mne->{conf}->{webdir}/img/topics";
    }
    if (!$imgdir)
    {
	return;
    }
    my $tmpl = $self->getTemplate();
    if ($self->{topicicon} && $tmpl->{link_icon})
    {
	my $http = new HTTP::Lite();
	$http->add_req_header("Accept-Language", $tmpl->{lang});
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request(sprintf($tmpl->{link_icon}, $self->{topicicon}));
	if ($req)
	{
	    open(IMG, ">$imgdir/$self->{topic}.png");
	    print IMG $http->body();
	    close(IMG);
	    chmod(0664, "$imgdir/$self->{topic}.png");
	}	
    }
}

sub checkDate # check date and time fields of the news and fill them if necessary
{
    my $self = shift;

    my @tm = localtime();

    if (!$self->{date})
    {
	$self->{date} = sprintf("%04d-%02d-%02d", $tm[5]+1900, $tm[4]+1, $tm[3]);
    }
    if (!$self->{crtime})
    {
	$ms++;
	$self->{crtime} = sprintf("%02d:%02d:%02d.%02d : ", $tm[2], $tm[1], $tm[0], $ms);
    }
}

sub getMNURL # get the URL to the corresponding news in the MyNews website;
{
    my $self = shift;
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    return "$mne->{conf}->{weburl}/news.cgi?id=$self->{id}";
}

sub getWebURL # get the URL to the corresponding news on the web;
{
    my $self = shift;
    my $tmpl = $self->getTemplate();
    my $url = ".";
    if ($tmpl->{link_news})
    {
	my @fds = $tmpl->{link_news} =~ /(\%s)/g;
	if (scalar(@fds) == 2)
	{
	    my $tid = $self->{topic};
	    $tid =~ s/^[^_]+_//;
	    $url = sprintf($tmpl->{link_news}, $tid, $self->getNewsId());
	}
	elsif (scalar(@fds) == 1)
	{
	    $url = sprintf($tmpl->{link_news}, $self->getNewsId());
	}
    }
    return $url;
}

sub getDate # get the news date formatted according to the user language
{
    my $self = shift;

    my ($day, $month, $year, $century);

    if ($self->{date} =~ /(\d\d)(\d\d)-(\d\d)-(\d\d)/)
    {
	my $hidden = "";
	my $user = MN::CGI::getUser();
	my $mne = MN::Engine::getCurrent(); # MyNews Engine
	if ($user)
	{
	    $hidden = $user->isHidden("Century");
	}
	else
	{
	    $hidden = $mne->isHidden("Century");
	}
	$century = $1;
	$year = $2;
	$month = $3;
	$day = $4;
	if ($user && $user->{lang} eq "fr")
	{
	    return sprintf('%02d/%02d/<div class="century%s">%02d</div>%02d', $day, $month, $hidden, $century, $year);
	}
	else
	{
	    return sprintf('%02d/%02d/<div class="century%s">%02d</div>%02d', $month, $day, $hidden, $century, $year);
	}
    }
    else
    {
	return "";
    }
}

sub getTime # get the news time formatted
{
    my $self = shift;

    my ($hour, $min, $sec);
    if ($self->{crtime} =~ /(\d\d):(\d\d):(\d\d)/)
    {
	$hour = $1;
	$min = $2;
	$sec = $3;
	my $hidden = "";
	my $user = MN::CGI::getUser();
	my $mne = MN::Engine::getCurrent(); # MyNews Engine
	if ($user)
	{
	    $hidden = $user->isHidden("Seconds");
	}
	else
	{
	    $hidden = $mne->isHidden("Seconds");
	}
	return sprintf('%02d:%02d<div class="seconds%s">:%02d</div>', $hour, $min, $hidden, $sec);
    }
    else
    {
	return "";
    }
}

sub removetag # store a tag in @tags
{
    my $tag = shift;
    push @tags, $tag;
    return "<>";
}

sub addtag # get the first tag from @tags
{
    my $tag = shift;
    $tag = shift @tags;
    return $tag;
}

sub getCode # get HTML code for the news;
{
    my $self = shift;
    my $number = shift;
    my $templatecss = shift;
    my $topiccss = shift;
    my $test = shift; # 1 to display news in test mode
    my $search = shift; # 1 to display news search mode

    my $tmpl = $self->getTemplate();
    my $user = MN::CGI::getUser();
    my $mne = MN::Engine::getCurrent(); # MyNews Engine

    my $code = "<table class=\"news\" cellspacing=0 cellpadding=0 border=0>\n";
    $code .= "<tr class=\"newsheadertr\">\n";
    $code .= "<td class=\"newsheaderleft\">\n";
    $code .= "</td>\n";
    $code .= "<td class=\"newsheadertd\">\n";
    $code .= "<div class=\"newsheaderstart\"></div>\n";
    if ($number)
    {
	$code .= "<div class=\"newsnumber\">\n";
	$code .= '<a href="'.$self->getMNURL().'">'.$number."</a>";
	$code .= "</div>\n";
    }
    $code .= "<div class=\"newstitle\">\n";
    my $name = $self->{name};
    if ($search)
    {
	my $key;
	foreach $key (@_)
	{
	    $name =~ s/($key)/<font class="keyword">$1<\/font>/ig;
	}   
    }
    $code .= '<a href="'.$self->getWebURL().'">'.$name."</a>\n";
    $code .= "</div>\n";
    $code .= "<div class=\"newsheaderend\"></div>\n";
    $code .= "</td>\n";
    $code .= "<td class=\"newsheaderright\">\n";
    $code .= "</td>\n";
    $code .= "</tr>\n";

    $code .= "<tr class=\"newsbodytr\">\n";
    $code .= "<td class=\"newsbodyleft\">\n";
    $code .= "</td>\n";
    $code .= "<td class=\"newsbodytd\">\n";
    $code .= $tmpl->getCode($templatecss, "", $test);
    if ($self->{topic})
    {
	my $topic;
	if ($test)
	{
	    $topic = new MN::Topic();
	    $topic->{id} = $self->{topic};
	    $topic->{name} = $self->{topicname};
	    $topic->{iconfile} = $self->{topicicon};
	}
	else
	{
	    $topic = $self->getTopic();
	}
	if ($topic)
	{
	    $code .= $topic->getCode($topiccss, 0, $test);
	}
    }
    $code .= "<div class=\"newsbody\">\n";
    my $body = $self->{body};
    if ($search > 1)
    {
	@tags = ();
	$body =~ s/(<[^>]+>)/removetag($1)/mge;
	my $key;
	foreach $key (@_)
	{
	    $body =~ s/($key)/<font class="keyword">$1<\/font>/ig;
	}   
	$body =~ s/(<>)/addtag($1)/mge;
    }
    $code .= $body."\n";
    $code .= "</div>\n";
    $code .= "</td>\n";
    $code .= "<td class=\"newsbodyright\">\n";
    $code .= "</td>\n";
    $code .= "</tr>\n";

    $code .= "<tr class=\"newsfootertr\">\n";
    $code .= "<td class=\"newsfooterleft\">\n";
    $code .= "</td>\n";
    $code .= "<td class=\"newsfootertd\">\n";
    my $hidden;
    if ($user)
    {
	$hidden = $user->isHidden("Date");
    }
    else
    {
	$hidden = $mne->isHidden("Date");
    }
    $code .= "<div class=\"newsdate$hidden\">\n";
    $code .= $self->getDate()."\n";
    $code .= "</div>\n";
    if ($user)
    {
	$hidden = $user->isHidden("Time");
    }
    else
    {
	$hidden = $mne->isHidden("Time");
    }
    $code .= "<div class=\"newstime$hidden\">\n";
    $code .= $self->getTime()."\n";
    $code .= "</div>\n";
    if ($user)
    {
	$hidden = $user->isHidden("Flag");
    }
    else
    {
	$hidden = $mne->isHidden("Flag");
    }
    $code .= "<div class=\"newsflag$hidden\">\n";
    $code .= $tmpl->getFlag()."\n";
    $code .= "</div>\n";
    $code .= "</td>\n";
    $code .= "<td class=\"newsfooterright\">\n";
    $code .= "</td>\n";
    $code .= "</tr>\n";

    $code .= "</table>\n";

    return $code; # news HTML code
}

1;

__END__

=pod

=head1 NAME

MN::News - MyNews class to manage news.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 my $news = MN::News->getObject("slashdot_04/09/29/0112235");
 ...

=head1 DESCRIPTION

MyNews News objects store informations about collected news.

MN::News inherits from MN::Object, that means that news objects are stored in database.

=over 4

=head2 Fields

=item id : string

News id (primary key). 

=item name : string

News title as it appears in the website.

=item date : date

News date (extracted from the website or set to the date when the news was collected).

=item crtime : string

News creation time. Useful to sort news of the same day.

=item template : string

Template corresponding to the news. For example: "slashdot".

=item topic : string

When template manage topics, the topic field contains the id of the topic of the news. For example: "slashdot_222".

=item body : string

Content of the news as it appears in the website.

=head2 News information

=item getNewsId ()

Return the news id without template prefix. For example: "04/09/29/0112235"

=item getTemplate ()

Return template object corresponding to the news. Avoid database access if the template object has been already selected.

=item getTopic ()

Return topic object corresponding to the news. Avoid database access if the topic object has been already selected.

=item getMNURL ()

Return the URL of the news in the MyNews website.

=item getWebURL ()

Return the URL of the news in the original website.

=item getDate ()

Return the news date formatted according to the user language and display.

=item getTime ()

Return the news time formatted according to the user display.

=item getCode ([$number], [$templatecss], [$topiccss], [$test], [$search, keyword1, $keyword2...])

Return the HTML code to display the news in the MyNews website.

$number is the optional number to display in the news header.

$templatecss and $topiccss are the suffixes to add to the template and topic CSS classes.

If $test is set to 1, display the news in test mode (use remote icons for template and topic).

If $search is set to 1, highlight keywords in news title. $search is set to 2, highlight keywords in news title and body.

=head2 News extraction

=item extract ([$idonly])

Extract news data from news lines.

If $idonly is set to 1, only extract news id.

=item getTopicIcon ()

Extract icon of the news topic.

=item checkDate ()

Check wether news date and time field is set or not. If not, set them to now.

=head1 SEE ALSO

L<MN::Object>, L<MN::Template>, L<MN::Topic>
