package MN::UserGroup; # class to manage MyNews user/group links

use strict;

@MN::UserGroup::ISA = qw(MN::Object);

use MN::Object;

sub new # constructor
{
    my $class_name = shift;

    my $self = new MN::Object();
    
    bless $self, $class_name;

    return $self; # new UserGroup object
}
sub getClass # get database table for the class 
{
    return "mnusergroup"; # "mnuser"
}

sub getKey # get database key for the class
{
    return "login,groupid"; # "login,groupid"
}

sub getFields # get database fields for the class
{
    return { # fields hash table
	"login" => "string",
	"groupid" => "string",
    };
}

1;

__END__

=pod

=head1 NAME

MN::UserGroup - MyNews class to manage user/group links.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 my $user = $cgi->getUser();
 my @usgrs = MN::UserGroup->getObjects("login = '$user->{login}'");
 ...

=head1 DESCRIPTION

MyNews UserGroup objects store links between users and groups. There is only one group for the moment called "admin". It is used to define admin privilege to users.

MN::UserGroup inherits from MN::Object, that means that user/group links are stored in database.

=over 4

=head2 Fields

=item groupeid : string

Group id.

=item login : string

User login.

=head1 SEE ALSO

L<MN::Object>, L<MN::User>
